package ru.ulstu.conference.controller;


import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.Errors;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import ru.ulstu.conference.model.ConferenceDto;
import ru.ulstu.conference.model.ConferenceFilterDto;
import ru.ulstu.conference.service.ConferenceService;
import ru.ulstu.deadline.model.Deadline;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.io.IOException;
import java.util.stream.Collectors;

import static org.springframework.util.StringUtils.isEmpty;
import static ru.ulstu.core.controller.Navigation.CONFERENCES_PAGE;
import static ru.ulstu.core.controller.Navigation.CONFERENCE_PAGE;
import static ru.ulstu.core.controller.Navigation.REDIRECT_TO;


@Controller()
@RequestMapping(value = "/conferences")
@ApiIgnore
public class ConferenceController {

    private final ConferenceService conferenceService;

    public ConferenceController(ConferenceService conferenceService) {
        this.conferenceService = conferenceService;
    }

    @GetMapping("/conferences")
    public void getConferences(ModelMap modelMap) {
        modelMap.put("filteredConferences", new ConferenceFilterDto(conferenceService.findAllDto()));
    }

    @GetMapping("/conference")
    public void getConference(ModelMap modelMap, @RequestParam(value = "id") Integer id) {
        if (id != null && id > 0) {
            modelMap.put("conferenceDto", conferenceService.findOneDto(id));
        } else {
            modelMap.put("conferenceDto", new ConferenceDto());
        }
    }

    @PostMapping(value = "/conference", params = "save")
    public String save(@Valid ConferenceDto conferenceDto, Errors errors) throws IOException {
        filterEmptyDeadlines(conferenceDto);
        if (conferenceDto.getDeadlines().isEmpty()) {
            errors.rejectValue("deadlines", "errorCode", "Не может быть пустым");
        }
        if (errors.hasErrors())
            return CONFERENCE_PAGE;
        conferenceService.save(conferenceDto);
        return String.format(REDIRECT_TO, CONFERENCES_PAGE);

    }

    @PostMapping(value = "/conference", params = "addDeadline")
    public String addDeadline(@Valid ConferenceDto conferenceDto, Errors errors) {
        filterEmptyDeadlines(conferenceDto);
        if (errors.hasErrors())
            return CONFERENCE_PAGE;
        conferenceDto.getDeadlines().add(new Deadline());
        return CONFERENCE_PAGE;
    }

    private void filterEmptyDeadlines(ConferenceDto conferenceDto) {
        conferenceDto.setDeadlines(conferenceDto.getDeadlines().stream()
                .filter(dto -> dto.getDate() != null || !isEmpty(dto.getDescription()))
                .collect(Collectors.toList()));
    }
}
